##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################
import os, urllib, string
from Globals import DTMLFile, package_home, InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager
from TAL.TALDefs import TALESError
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFCore.PortalContent import NoWL, ResourceLockedError
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.utils import keywordsplitter
from Products.CMFDefault.Document import Document
from Products.PageTemplates import PageTemplate
from Products.PageTemplates.ZopePageTemplate import SecureModuleImporter
from Acquisition import Implicit, aq_parent, aq_inner
from Persistence import Persistent
from Products.CMFDefault.utils import *
from ExtensionClass import Base

_dtmldir = os.path.join( package_home( globals() ), 'dtml' )

# Import permission names
from Products.CMFCore import CMFCorePermissions
import DynamicDocumentPermissions

# Factory type information -- makes Events objects play nicely
# with the Types Tool (portal_types)
factory_type_information = (
    {'id': 'DynamicDocument',
     'content_icon': 'document_icon.gif',
     'meta_type': 'CMF Dynamic Document',
     'description': ('Dynamic Documents allow content authors to include TALES with content.'),
     'product': 'CMFDynamicDocument',
     'factory': 'addDynamicDocument',
     'immediate_view': 'metadata_edit_form',
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'document_view',
                  'permissions': (CMFCorePermissions.View,)},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'ddocument_edit_form',
                  'permissions': (DynamicDocumentPermissions.ChangeDynamicDocuments,)},
                 {'id': 'metadata'
                 , 'name': 'Metadata'
                 , 'action': 'metadata_edit_form'
                 , 'permissions': (DynamicDocumentPermissions.ChangeDynamicDocuments,)},
                 ),                     # End Actions
     },
    )

def addDynamicDocument(self
             , id
             , title=''
             , description=''
             , text=''):
    """
    Create an empty document.
    """
    dd = DynamicDocument(id
                  , title
                  , description
                  , text
                 )
    self._setObject(id, dd)

class ddPageTemplate(PageTemplate.PageTemplate, Implicit, Persistent):
    def pt_getContext(self):
        root = self.getPhysicalRoot()
        template = aq_parent(self)
        here = aq_parent(template)
        c = {'template': template,
             'options': {},
             'nothing': None,
             'request': None,
             'root': root,
             'request': getattr(root, 'REQUEST', None),
             'modules': SecureModuleImporter,
             'here': here,
             'container': aq_inner(here),
             }
        return c

    def __setstate__(self, state):
        ddPageTemplate.inheritedAttribute('__setstate__')(self, state)
        self._cook()

class Macros(Base):
    """
    Evaluate to the macros collection of the contained page template.
    """
    def __of__( self, doc):
        return doc.get_pt().macros

class DynamicDocument(Document):
    """
    DynamicDocuments are CMFDocuments merged with the ability to write TALES
    expressions.
    """
    __implements__ = (Document.__implements__,)
    meta_type='CMF Dynamic Document'

    macros = Macros()
    
    # Declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)
    
    def __init__(self
                 , id
                 , title=''
                 , description=''
                 , text='' 
                ):
        self._pt=ddPageTemplate()
        DefaultDublinCoreImpl.__init__(self)
        self.id=id
        self.setTitle(title)
        self.setDescription(description)
        self._pt.write(text)
        self.edit(text=text)
    
    security.declareProtected(CMFCorePermissions.View, 'macros')
    
    security.declareProtected(CMFCorePermissions.ModifyPortalContent,
                                  'manage_edit')
    manage_edit = DTMLFile('zmi_editDynamicDocument', _dtmldir)

    def get_pt(self):
        """
            return wrapped PageTemplate object.
        """
        return self._pt.__of__(self)

    def manage_editDocument(self, text, file='', REQUEST=None):
        """ 
          A ZMI (Zope Management Interface) level editing method.
        """
        self._edit(text, file)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()
                                         + '/manage_edit'
                                         + '?manage_tabs_message=Document+updated'
                                                                                                            )
    
    
    def _edit(self, text, safety_belt=''):    
        """
          Edit the document
        """
        pt = self.get_pt()
        pt.write(text)
        headers = {}
        if not safety_belt:
            safety_belt=headers.get('SafetyBelt', '')
        if not self._safety_belt_update(safety_belt=safety_belt):
            msg = ("Intervening changes from elsewhere were detected."
                   "  Please refetch the document and reapply your changes."
                   "  you may be able to recover your version by using the "
                   "browser back button.  You will have to reapply them to "
                   "a freshly fetched copy.")
            raise 'EditingConflict', msg
        pt.write(text)

    security.declareProtected(DynamicDocumentPermissions.ChangeDynamicDocuments, 'edit')
    def edit( self
            , text
            , file=''
            , safety_belt=''
            ):
        """
        *used to be WorkflowAction(_edit)
        To add webDav support, we need to check if the content is locked, and if
        so return ResourceLockedError if not, call _edit.

        Note that this method expects to be called from a web form, and so
        disables header processing
        """
        self.failIfLocked()
        if file and (type(file) is not type('')):
            contents=file.read()
            if contents:
                text = pt.write(contents) 
        text = bodyfinder(text)
        self._edit(text=text, safety_belt=safety_belt)
        self.reindexObject()

    
    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP (and presumably FTP?) PUT requests """
        if not NoWL:
            self.dav__init(REQUEST, RESPONSE)
            self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        body = REQUEST.get('BODY', '')
        self.setFormat('text/html')
        try:
            headers, body, format = self.handleText(text=body)
            safety_belt = headers.get('SafetyBelt', '')
            self.setMetadata(headers)
            self._edit(text=body, safety_belt=safety_belt)
        except 'EditingConflict', msg:
            get_transaction().abort()
            RESPONSE.setStatus(450)
            return RESPONSE
        except ResourceLockedError, msg:
            get_transaction().abort()
            RESPONSE.setStatus(423)
            return RESPONSE
        RESPONSE.setStatus(204)
        self.reindexObject()
        return RESPONSE

    def SearchableText(self):
        """
           Used by the catalog for basic full text indexing.
        """
        return "%s %s %s" % (self.title, self.description, self.get_pt().read())

    def CookedBody(self):
        """
           Return a rendering of the Dynamic Document.
        """
        security = getSecurityManager()
        security.addContext(self)
        try:
            try:
                result = self.get_pt().pt_render(
                    extra_context={'user': security.getUser()})
            except TALESError, err:
                if err.type == 'Unauthorized':
                    raise err.type, err.value, err.traceback
                raise
            return result
        finally:
            security.removeContext(self)

    def EditableBody(self):
        """
          The editable body of text.  This is the raw structured text 
          or HTML.
        """
        return self.get_pt().read()

    def Format(self):
        """
            Returns a content-type style format of the underlying source
        """
        return self._pt.content_type

    def setFormat(self, value):
        """
           Ignore the value, as we only support text/html currently
        """
        self._pt.content_type = 'text/html'
    setFormat = WorkflowAction(setFormat)

# Intialize the DynamicDocument class, setting up security.
InitializeClass(DynamicDocument)
